/* player.c setup, update and rendering of the player
 *
 * Copyright (c) 2019, Chris Camacho (codifies) http://bedroomcoders.co.uk/captcha/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "stdlib.h"

#include "raylib.h"

#include "player.h"
#include "aliens.h"
#include "screens.h"
#include "game.h"


// prototype from game unit
void setGameExit(bool b);

// the player can only do things depending on the player mode
void updatePlayer()
{
    // TODO add settings screen with settings for key bindings
    // only allow the player to move if in playing mode
    if (player.mode == pm_Playing) {

       if (IsKeyDown(KEY_Z)) {
            player.pos.x -= 8;
            if (player.pos.x < 32) {
                player.pos.x = 32;
            }
        }

        if (IsKeyDown(KEY_X)) {
            player.pos.x += 8;
            if (player.pos.x > gameScreenWidth-32) {
                player.pos.x = gameScreenWidth-32;
            }
        }

        // TODO add shooting here
    }


    if (player.mode == pm_GameOver) {
        player.frame++;
        if (player.frame>240) {
            // delay allows player to read the game over message
            // also allow the score to increase if existing shots
            // kill further aliens...
            // optionally you could retire all the player shots here...
            setGameExit(true);
        }
    }

    if (player.mode == pm_Explode) {
        if (player.frame==0) {
            PlaySoundEx(explosionSound);
        }
        player.frame++;
        if (player.frame>120) { // allow for a short pause after
            player.lives--;
            if (player.lives<1) {
                player.mode = pm_GameOver;
                player.frame = 0;
            } else {
                // just reset player mode for next life
                // could add extra modes to handle this better...

                // TODO need to reset all the shots here
                player.mode = pm_Playing;
            }
        }
    }

}

// draws the player and also the lives indicator
void drawPlayer()
{
    Rectangle src = (Rectangle){0,0,64,64};
    Rectangle dst = (Rectangle){0,0,64,64};
    Vector2 oset = (Vector2){32,32};

    // draw player
    dst.x = player.pos.x;
    dst.y = player.pos.y;
    if (player.mode != pm_GameOver) {
        if (player.mode!=pm_Explode) {
            DrawTexturePro(player.tx, src, dst, oset, 0, WHITE);
        } else {
            if (player.frame<25) {
                // don't draw after the explostion has ended
                src = (Rectangle){player.frame*64,0,64,64};
                DrawTexturePro(explosion, src, dst, oset, 0, WHITE);
                src = (Rectangle){0,0,64,64};
            }
        }
    }

    // draw player lives
    dst.y = gameScreenHeight - 18;
    dst.width = dst.height = 32;
    for (int i=0; i<player.lives-1; i++) {
        dst.x = i * 34 + 48;
        DrawTexturePro(player.tx, src, dst, (Vector2){16,16}, 0, WHITE);
    }
    dst.width = dst.height = 64;

}
